"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sizeCheckBundles = void 0;
const cosmiconfig_1 = require("cosmiconfig");
const globby_1 = __importDefault(require("globby"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const multimatch_1 = __importDefault(require("multimatch"));
const filesize_1 = __importDefault(require("filesize"));
const bytes_1 = require("bytes");
const picocolors_1 = __importDefault(require("picocolors"));
function sizeCheckBundles({ outDir }) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        // get config
        const explorer = (0, cosmiconfig_1.cosmiconfig)('in-it');
        const result = yield explorer.search();
        if (!result || result.isEmpty) {
            return;
        }
        const trackGlob = (_a = result.config.track) !== null && _a !== void 0 ? _a : '**/*';
        const inItConfig = result.config;
        // get with globy the relevant files
        const trackedFiles = yield (0, globby_1.default)(trackGlob, {
            cwd: outDir,
        });
        // get the size of each file
        const fileSizes = {};
        yield Promise.all(trackedFiles.map((file) => __awaiter(this, void 0, void 0, function* () {
            const fileFullPath = path_1.default.join(outDir, file);
            const fileSize = (yield fs_extra_1.default.stat(fileFullPath)).size;
            fileSizes[file] = fileSize;
        })));
        const errors = [];
        const matchedFiles = new Set();
        // compare to hard coded limits
        (_b = inItConfig.limits) === null || _b === void 0 ? void 0 : _b.forEach((limit) => {
            Object.entries(limit).forEach(([limitedGlobby, limit]) => {
                const matches = (0, multimatch_1.default)(trackedFiles, limitedGlobby);
                matches
                    .filter((file) => !matchedFiles.has(file))
                    .forEach((file) => {
                    matchedFiles.add(file);
                    const fileSize = fileSizes[file];
                    if (limit.maxSize && fileSize >= (0, bytes_1.parse)(limit.maxSize)) {
                        errors.push(`File ${picocolors_1.default.bold(file)} size is ${picocolors_1.default.bold((0, filesize_1.default)(fileSize))}, which is more than the limit of ${picocolors_1.default.bold(limit.maxSize)}`);
                    }
                });
            });
        });
        // TODO send result to github status check
        // if there are any relative limits, compare to the previous\master build
        // gather all errors and print them
        // if there are any errors, exit with error code
        if (errors.length) {
            console.error(errors.join('\n'));
            console.error(picocolors_1.default.red('Size check failed'));
            // throw new Error('Size check failed');
            process.exit(1);
        }
    });
}
exports.sizeCheckBundles = sizeCheckBundles;
